@@ -1,6 +1,6 @@
 # Dynamic architecture support for GDB, the GNU debugger.
 
-# Copyright (C) 1998-2024 Free Software Foundation, Inc.
+# Copyright (C) 1998-2025 Free Software Foundation, Inc.
 
 # This file is part of GDB.
 
@@ -1431,12 +1431,12 @@ Function(
     invalid=False,
 )
 
-Value(
-    comment="Vtable of solib operations functions.",
-    type="const solib_ops *",
-    name="so_ops",
-    predefault="&solib_target_so_ops",
-    printer="host_address_to_string (gdbarch->so_ops)",
+Function(
+    comment="Return a newly-allocated solib_ops object capable of providing the solibs for this architecture.",
+    type="solib_ops_up",
+    name="make_solib_ops",
+    params=[],
+    predefault="make_target_solib_ops",
     invalid=False,
 )
 
@@ -1498,7 +1498,7 @@ Function(
     comment="""
 Process an ELF symbol in the minimal symbol table in a backend-specific
 way.  Normally this hook is supposed to do nothing, however if required,
-then this hook can be used to apply tranformations to symbols that are
+then this hook can be used to apply transformations to symbols that are
 considered special in some way.  For example the MIPS backend uses it
 to interpret `st_other' information to mark compressed code symbols so
 that they can be treated in the appropriate manner in the processing of
@@ -1506,7 +1506,7 @@ the main symbol table and DWARF-2 records.
 """,
     type="void",
     name="elf_make_msymbol_special",
-    params=[("asymbol *", "sym"), ("struct minimal_symbol *", "msym")],
+    params=[("const asymbol *", "sym"), ("struct minimal_symbol *", "msym")],
     predicate=True,
 )
 
@@ -1522,7 +1522,7 @@ Function(
     comment="""
 Process a symbol in the main symbol table in a backend-specific way.
 Normally this hook is supposed to do nothing, however if required,
-then this hook can be used to apply tranformations to symbols that
+then this hook can be used to apply transformations to symbols that
 are considered special in some way.  This is currently used by the
 MIPS backend to make sure compressed code symbols have the ISA bit
 set.  This in turn is needed for symbol values seen in GDB to match
@@ -2126,7 +2126,7 @@ Record architecture-specific information from the symbol table.
 """,
     type="void",
     name="record_special_symbol",
-    params=[("struct objfile *", "objfile"), ("asymbol *", "sym")],
+    params=[("struct objfile *", "objfile"), ("const asymbol *", "sym")],
     predicate=True,
 )
 
@@ -2224,7 +2224,7 @@ For example, on x86 the register indirection is written as:
 
 (%eax) ;; indirecting eax
 
-in this case, this prefix would be the charater `('.
+in this case, this prefix would be the character `('.
 
 Please note that we use the indirection prefix also for register
 displacement, e.g., `4(%eax)' on x86.
@@ -2243,7 +2243,7 @@ For example, on x86 the register indirection is written as:
 
 (%eax) ;; indirecting eax
 
-in this case, this prefix would be the charater `)'.
+in this case, this prefix would be the character `)'.
 
 Please note that we use the indirection suffix also for register
 displacement, e.g., `4(%eax)' on x86.
@@ -2552,7 +2552,7 @@ Implement the "info proc" command.
 
 Method(
     comment="""
-Implement the "info proc" command for core files.  Noe that there
+Implement the "info proc" command for core files.  Note that there
 are two "info_proc"-like methods on gdbarch -- one for core files,
 one for live targets.
 """,
